/*
 * jQuery treeTable Plugin unreleased
 * http://ludo.cubicphuse.nl/jquery-plugins/treeTable/doc/
 *
 * Copyright 2011, Ludo van den Boom
 * Dual licensed under the MIT or GPL Version 2 licenses.
 */

(function($) {
  // Helps to make options available to all functions
  // TODO: This gives problems when there are both expandable and non-expandable
  // trees on a page. The options shouldn't be global to all these instances!
  var options;
  var defaultPaddingLeft;

  $.fn.treeTable = function(opts) {
    options = $.extend({}, $.fn.treeTable.defaults, opts);

    return this.each(function() {
      $(this).addClass("treeTable").find("tbody tr").each(function() {
        // Initialize root nodes only if possible
        if(!options.expandable || $(this)[0].className.search(options.childPrefix) == -1) {
          // To optimize performance of indentation, I retrieve the padding-left
          // value of the first root node. This way I only have to call +css+
          // once.
          if (isNaN(defaultPaddingLeft)) {
            defaultPaddingLeft = parseInt($($(this).children("td")[options.treeColumn]).css('padding-left'), 10);
          }

          initialize($(this));
        } else if(options.initialState == "collapsed") {
          this.style.display = "none"; // Performance! $(this).hide() is slow...
        }
      });
    });
  };

  $.fn.treeTable.defaults = {
    childPrefix: "child-of-",
    clickableNodeNames: false,
    expandable: true,
    indent: 19,
    initialState: "collapsed",
    onNodeShow: null,
    treeColumn: 0
  };

  // Recursively hide all node's children in a tree
  $.fn.collapse = function() {
    $(this).addClass("collapsed");

    childrenOf($(this)).each(function() {
      if(!$(this).hasClass("collapsed")) {
        $(this).collapse();
      }

      this.style.display = "none"; // Performance! $(this).hide() is slow...
    });

    return this;
  };

  // Recursively show all node's children in a tree
  $.fn.expand = function() {
    $(this).removeClass("collapsed").addClass("expanded");

    childrenOf($(this)).each(function() {
      initialize($(this));

      if($(this).is(".expanded.parent")) {
        $(this).expand();
      }

      // this.style.display = "table-row"; // Unfortunately this is not possible with IE :-(
      $(this).show();

      if($.isFunction(options.onNodeShow)) {
        options.onNodeShow.call();
      }
    });

    return this;
  };

  // Reveal a node by expanding all ancestors
  $.fn.reveal = function() {
    $(ancestorsOf($(this)).reverse()).each(function() {
      initialize($(this));
      $(this).expand().show();
    });

    return this;
  };

  // Add an entire branch to +destination+
  $.fn.appendBranchTo = function(destination) {
    var node = $(this);
    var parent = parentOf(node);

    var ancestorNames = $.map(ancestorsOf($(destination)), function(a) { return a.id; });

    // Conditions:
    // 1: +node+ should not be inserted in a location in a branch if this would
    //    result in +node+ being an ancestor of itself.
    // 2: +node+ should not have a parent OR the destination should not be the
    //    same as +node+'s current parent (this last condition prevents +node+
    //    from being moved to the same location where it already is).
    // 3: +node+ should not be inserted as a child of +node+ itself.
    if($.inArray(node[0].id, ancestorNames) == -1 && (!parent || (destination.id != parent[0].id)) && destination.id != node[0].id) {
      indent(node, ancestorsOf(node).length * options.indent * -1); // Remove indentation

      if(parent) { node.removeClass(options.childPrefix + parent[0].id); }

      node.addClass(options.childPrefix + destination.id);
      move(node, destination); // Recursively move nodes to new location
      indent(node, ancestorsOf(node).length * options.indent);
    }

    return this;
  };

  // Add reverse() function from JS Arrays
  $.fn.reverse = function() {
    return this.pushStack(this.get().reverse(), arguments);
  };

  // Toggle an entire branch
  $.fn.toggleBranch = function() {
    if($(this).hasClass("collapsed")) {
      $(this).expand();
    } else {
      $(this).removeClass("expanded").collapse();
    }

    return this;
  };

  // === Private functions

  function ancestorsOf(node) {
    var ancestors = [];
    while(node = parentOf(node)) {
      ancestors[ancestors.length] = node[0];
    }
    return ancestors;
  };

  function childrenOf(node) {
    return $("table.treeTable tbody tr." + options.childPrefix + node[0].id);
  };

  function getPaddingLeft(node) {
    var paddingLeft = parseInt(node[0].style.paddingLeft, 10);
    return (isNaN(paddingLeft)) ? defaultPaddingLeft : paddingLeft;
  }

  function indent(node, value) {
    var cell = $(node.children("td")[options.treeColumn]);
    cell[0].style.paddingLeft = getPaddingLeft(cell) + value + "px";

    childrenOf(node).each(function() {
      indent($(this), value);
    });
  };

  function initialize(node) {
    if(!node.hasClass("initialized")) {
      node.addClass("initialized");

      var childNodes = childrenOf(node);

      if(!node.hasClass("parent") && childNodes.length > 0) {
        node.addClass("parent");
      }

      if(node.hasClass("parent")) {
        var cell = $(node.children("td")[options.treeColumn]);
        var padding = getPaddingLeft(cell) + options.indent;

        childNodes.each(function() {
          $(this).children("td")[options.treeColumn].style.paddingLeft = padding + "px";
        });

        if(options.expandable) {
          cell.prepend('<span style="margin-left: -' + options.indent + 'px; padding-left: ' + options.indent + 'px" class="expander"></span>');
          $(cell[0].firstChild).click(function() { node.toggleBranch(); });

          if(options.clickableNodeNames) {
            cell[0].style.cursor = "pointer";
            $(cell).click(function(e) {
              // Don't double-toggle if the click is on the existing expander icon
              if (e.target.className != 'expander') {
                node.toggleBranch();
              }
            });
          }

          // Check for a class set explicitly by the user, otherwise set the default class
          if(!(node.hasClass("expanded") || node.hasClass("collapsed"))) {
            node.addClass(options.initialState);
          }

          if(node.hasClass("expanded")) {
            node.expand();
          }
        }
      }
    }
  };

  function move(node, destination) {
    node.insertAfter(destination);
    childrenOf(node).reverse().each(function() { move($(this), node[0]); });
  };

  function parentOf(node) {
    var classNames = node[0].className.split(' ');

    for(var key=0; key<classNames.length; key++) {
      if(classNames[key].match(options.childPrefix)) {
        return $("#" + classNames[key].substring(9));
      }
    }

    return null;
  };
})(jQuery);
/*
 * Copyright (c) 2008 Greg Weber greg at gregweber.info
 * Dual licensed under the MIT and GPLv2 licenses just as jQuery is:
 * http://jquery.org/license
 *
 * documentation at http://gregweber.info/projects/uitablefilter
 *
 * allows table rows to be filtered (made invisible)
 * <code>
 * t = $('table')
 * $.uiTableFilter( t, phrase )
 * </code>
 * arguments:
 *   jQuery object containing table rows
 *   phrase to search for
 *   optional arguments:
 *     column to limit search too (the column title in the table header)
 *     ifHidden - callback to execute if one or more elements was hidden
 *
 * If a user has a table where certain rows should not be filtered,
 * simply add a class of 'not_filtered' to those rows and they will
 * be skipped during filtering.
 */

(function($) {
  $.uiTableFilter = function(jq, phrase, column, ifHidden){
    var new_hidden = false;
    if( this.last_phrase === phrase ) return false;

    var phrase_length = phrase.length;
    var words = phrase.toLowerCase().split(" ");

    // these function pointers may change
    var matches = function(elem) { elem.show() }
    var noMatch = function(elem) { elem.hide(); new_hidden = true }
    var getText = function(elem) { return elem.text() }

    if( column ) {
      var index = null;
      jq.find("thead > tr:last > th").each( function(i){
        if( $.trim($(this).text()) == column ){
          index = i; return false;
        }
      });
      if( index == null ) throw("given column: " + column + " not found")

      getText = function(elem){ return $(elem.find(
        ("td:eq(" + index + ")")  )).text()
      }
    }

    // if added one letter to last time,
    // just check newest word and only need to hide
    if( (words.size > 1) && (phrase.substr(0, phrase_length - 1) ===
          this.last_phrase) ) {

      if( phrase[-1] === " " )
      { this.last_phrase = phrase; return false; }

      var words = words[-1]; // just search for the newest word

      // only hide visible rows
      matches = function(elem) {;}
      var elems = jq.find("tbody:first > tr:visible").not(".not_filtered")
    }
    else {
      new_hidden = true;
      var elems = jq.find("tbody:first > tr").not(".not_filtered")
    }

    elems.each(function(){
      var elem = $(this);
      $.uiTableFilter.has_words( getText(elem), words, false ) ?
        matches(elem) : noMatch(elem);
    });

    last_phrase = phrase;
    if( ifHidden && new_hidden ) ifHidden();
    return jq;
  };

  // caching for speedup
  $.uiTableFilter.last_phrase = ""

  // not jQuery dependent
  // "" [""] -> Boolean
  // "" [""] Boolean -> Boolean
  $.uiTableFilter.has_words = function( str, words, caseSensitive )
  {
    var text = caseSensitive ? str : str.toLowerCase();
    for (var i=0; i < words.length; i++) {
      if (text.indexOf(words[i]) === -1) return false;
    }
    return true;
  }
}) (jQuery);
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/


KT.panel.list.registerPage('providers', { create : 'new_provider' });

$(document).ready(function() {

    KT.panel.set_expand_cb(function() {
        KT.object.label.initialize();
        if ($('#providers').length > 0) {
            notices.checkNotices();
        }

        if ($('#discovered_repos').length > 0){
            KT.repo_discovery.page_load();
        }
        else {
            KT.repo_discovery.page_close();
        }
    });

  $('.repo_create').live('click', function(event) {
    var button = $(this);
    button.addClass("disabled");

    event.preventDefault();
    var form = $(this).closest("form");
    var url = form.attr('action');
    var dataToSend = form.serialize();
    // send a request to create the repo
    client_common.create(dataToSend, url, function() {
        KT.panel.panelAjax('', button.attr("data-url") ,$('#panel'));
        KT.panel.closeSubPanel($('#subpanel'));
      },
      function() {
          button.removeClass("disabled");
    });
  });
  $('#provider_contents').attr('size', '17');
  //end doc ready
});

var provider = (function() {
    return {
        toggleFields : function() {
              var val = $('#provider_provider_type option:selected').val();
              var fields = "#repository_url_field";
              if (val === "Custom") {
                  $(fields).attr("disabled", true);
              }
              else {
                  $(fields).removeAttr("disabled");
              }
        }
    };
})();
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/




KT.repo_discovery = (function(){
    var updater,
        form_id = '#repo_discovery_form',
        list_id = '#discovered_repos';

    var page_load = function(){
        draw_url_list(KT.initial_repo_discovery.urls);
        if(KT.initial_repo_discovery.running){
            discovery_started();
            disable_discovery();
        }
        $(form_id).unbind('submit').submit(function(e){
            e.preventDefault();
            start_discovery();
        });

        $('#new_repos').unbind('click').click(open_subpane);

        $('#url_filter').bind('change, keyup', function(){
            $.uiTableFilter($(list_id), this.value, $(list_id).find("thead > tr:last > th").first().text().trim());
        });

        $(list_id).delegate("input[type=checkbox]", 'change', on_checkbox_change);
        on_checkbox_change();
        init_cancel();
    },
    open_subpane = function(){
        var urls = '?';
        KT.utils.each(selected(), function(element, index){
           if(index > 0) {
               urls += '&';
           }
           urls+= 'urls[]=' + element;
        });
        KT.panel.openSubPanel($(this).data('url') + urls );
    },
    page_close = function(){
        if(updater !== undefined){
            updater.stop();
            updater = undefined;
        }
    },
    init_updater = function(){
        var url = $(list_id).data('url');
        if(updater !== undefined){
            return;
        }
        updater = $.PeriodicalUpdater(url, {
              method: 'get',
              type: 'json',
              global: 'false'
            },
            function(data, success) {
                if(data !== '') { //403
                    draw_url_list(data.urls);
                    if(!data.running){
                        discovery_ended();
                    }
                }
            });
    },
    init_cancel = function(){
        var cancel = $(form_id).find('#cancel_discover');
        cancel.unbind('click');
        cancel.click(cancel_discovery);
    },
    draw_url_list = function(url_list){
        var list = $(list_id),
            find_text = function(){ return $(this).find('.hidden-text').html();};
        list.find('.check_icon-black').tipsy('hide');
        KT.initial_repo_discovery.urls = url_list;
        list.find('tbody').html(KT.discovery_templates.url_list(url_list, selected()));
        $('#url_filter').trigger('keyup');

        list.find('.check_icon-black').tipsy({html:true, gravity:'w', className:'content-tipsy',
            title:find_text});
    },
    start_discovery = function(){
        var form = $(form_id),
            discover_url = form.find('input[type=text]').val();

        disable_discovery();
        $.ajax({
            contentType:"application/json",
            type: "POST",
            url: form.data('url'),
            data: JSON.stringify({'url':discover_url}),
            cache: false,
            success: function(data) {
                discovery_started();
            },
            error: function(data) {
                enable_discovery();
            }
        });
        draw_url_list([]);
    },
    cancel_discovery = function(){
        var button = $(form_id).find('#cancel_discover');
        button.attr('disabled', 'disabled');
        $.ajax({
            contentType:"application/json",
            type: "POST",
            url: button.data('url'),
            cache: false,
            success: function(data) {
            },
            error: function(data) {
                button.removeAttr('disabled');
            }
        });

    },
    disable_discovery = function(){
        var form = $(form_id),
            cancel = form.find('#cancel_discover');
        form.find('input[type=text]').attr('disabled', 'disabled');
        form.find('input[type=submit]').parent().hide();
        cancel.removeAttr('disabled');
        cancel.parent().show();
    },
    enable_discovery = function(){
        var form = $(form_id);
        form.find('input[type=text]').removeAttr('disabled');
        form.find('input[type=submit]').parent().show();
        form.find('#cancel_discover').parent().hide();
    },
    discovery_started = function() {
        $(list_id).find('tbody').html('<tr><td></td><td></td></tr>');
        $('#url_filter').val('');
        init_updater();
    },
    discovery_ended = function(){
        updater.stop();
        KT.initial_repo_discovery.running = false;
        updater = undefined;
        enable_discovery();
    },
    on_checkbox_change = function(){
        var count = selected().length;
        if(count === 0){
            $('#new_repos').attr('disabled', 'disabled');
        }else {
            $('#new_repos').removeAttr('disabled');
        }
    },
    selected = function(){
        var to_ret = [];
        KT.utils.each($(list_id).find(":checked"), function(element){
            to_ret.push($(element).val());
        });
        return to_ret;
    },
    clear_selections = function(){
        $(list_id).find(":checked").removeAttr('checked');
    };

    return {
        page_load: page_load,
        page_close: page_close,
        clear_selections: clear_selections,
        init_updater: init_updater
    };
})();


KT.discovery_templates = (function(){
    var url_list = function(url_list, selected_list){
        var html = '';

        if(selected_list === undefined){
            selected_list = [];
        }
        if (url_list.length === 0) {
            return '<tr><td></td><td></td></tr>';
        }
        KT.utils.each(url_list, function(elem, index){
            html += url_list_item(elem, selected_list, index % 2 === 0);
        });
        return html;
    },
    url_list_item = function(item, selected_list, odd){
        var selected = '',
            html = '',
            alt = '';
        if (!odd) {
            alt = 'alt';
        }
        if (KT.utils.indexOf(selected_list, item.url) !== -1){
            selected = 'checked';
        }
        html = '<tr class="' + alt + '"><td><label>';
        html += '<input type="checkbox"' + selected + ' value="' + item.url + '"/>' + item.path + '</label>';
        html += '</td>';
        html += '<td>' + existing_tipsy(item.existing) + '</td></tr>';
        return html;
    },
    existing_tipsy = function(existing_hash){
        var html = '';
        if (KT.utils.isEmpty(existing_hash)) {
            return html;
        }

        html += '<span class="grid_3"><span class="check_icon-black">';
        html += '<span class="hidden-text hidden"><span class="repo_tipsy la">';
        html += i18n.existing_repos_found + '<ul>';
        KT.utils.each(existing_hash, function(repo_list, product_name){
            html += existing_product(product_name, repo_list);
        });

        html += '</ul></span></span></span></span>';
        return html;
    },
    existing_product = function(product_name, repo_list){
        var html = '<li class="product_item">' + product_name + '<ul class="repo_list">';
        KT.utils.each(repo_list, function(repo_name){
           html += '<li class="repo_item">' + repo_name + '</li>';
        });

        return html + '</ul></li>';
    };

    return {
        url_list:url_list
    };

})();


KT.repo_discovery.new_page = (function(){
    var panel_id = '#repo_creation',
        product_select_id = "#existing_product_select",
        form_id = '#discovered_creation',
        product_details = '#product_details';


    var init_panel = function(){
        $(product_select_id).chosen();
        $(panel_id).find('input[type=radio]').change(radio_change);
        $(form_id).submit(submit_form);
        $(window).unbind('repo.create');
        $(window).bind('repo.create', create_repos);
    },
    submit_form = function(event) {
        event.preventDefault();
        var form = $(form_id),
            product_details = form.find('#product_details'),
            product_id, name, label, provider_id;

        disable_form();

        if (product_details.find('input[type=radio]:checked').val() === 'true'){
            name = product_details.find('input[type=text][name=product_name]').val();
            label = product_details.find('input[type=text][name=product_label]').val();
            create_product(name, label, $('#new_product').data('url'));
        }
        else {
            product_id = $(product_select_id).val();
            initiate_repo_creation(product_id);
        }

    },
    initiate_repo_creation = function(product_id) {
        var repos = [],
            provider_id = $(form_id).data('provider_id'),
            create_url = KT.routes.provider_product_repositories_path(provider_id, product_id);

        KT.utils.each($('.new_repo'), function(repo_div){
            repo_div = $(repo_div);
            var name = repo_div.find('.name_input').val(),
                label = repo_div.find('.label_input').val(),
                url = repo_div.find('input[type=hidden]').val(),
                id = '#' + repo_div.attr('id'),
                unprotected = $('#unprotected').find('input[type=checkbox]:checked').val() === '1';
            repos.push({name:name, label:label, feed:url, id:id, unprotected:unprotected});
        });
        $(window).trigger('repo.create', [create_url, repos]);
    },
    create_product = function(name, label, create_url){
        $.ajax({
            url:create_url,
            type: 'POST',
            data: {product:{name:name, label:label}},
            success: function(data){
                var product_div = $('#new_product');
                product_div.find('.name_input').replaceWith(name);
                product_div.find('.label_input').replaceWith(label);
                initiate_repo_creation(data.id);
            },
            error: function(){
                enable_form();
            }
        });
    },
    create_repos = function(event, create_url, repo_list){
        var repo = repo_list.shift();

        $.ajax({
            url:create_url,
            type: 'POST',
            data: {'repo':repo, 'ignore_success_notice':true},
            success:function(){
                var repo_div = $(repo.id),
                    created_num, created_msg;
                repo_div.removeClass('new_repo').addClass('created_repo');
                repo_div.find('.name_input').replaceWith(repo.name);
                repo_div.find('.label_input').replaceWith(repo.label);

                if (repo_list.length !== 0){
                    $(window).trigger('repo.create', [create_url, repo_list]);
                }
                else {
                    KT.repo_discovery.clear_selections();
                    KT.repo_discovery.init_updater();

                    created_num = $('.created_repo').length;
                    if (created_num === 1) {
                        created_msg = i18n.discovery_success_one;
                    }
                    else{
                        created_msg = i18n.discovery_success_multi(created_num);
                    }

                    KT.panel.closeSubPanel($('#subpanel'));
                    notices.displayNotice('success', JSON.stringify({notices:[created_msg]}),
                                            'repositories___create');

                }
            },
            error: function(){
                enable_form();
            }
        });


    },
    disable_form = function(){
        $(form_id).find('input').attr('disabled', 'disabled');
        $(product_select_id).attr('disabled', true).trigger("liszt:updated");
    },
    enable_form = function(){
        $(form_id).find('input').removeAttr('disabled');
        $(product_select_id).attr('disabled', false).trigger("liszt:updated");
    },
    radio_change = function(){
        if ($(this).val() === 'true'){
            $(product_select_id).attr('disabled', true).trigger("liszt:updated");
            $('#new_product').show();
        }
        else {
            $(product_select_id).attr('disabled', false).trigger("liszt:updated");
            $('#new_product').hide();
        }
    };



    return {
        init_panel:init_panel
    };
})();
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/





;
