/*
 * jQuery treeTable Plugin unreleased
 * http://ludo.cubicphuse.nl/jquery-plugins/treeTable/doc/
 *
 * Copyright 2011, Ludo van den Boom
 * Dual licensed under the MIT or GPL Version 2 licenses.
 */

(function($) {
  // Helps to make options available to all functions
  // TODO: This gives problems when there are both expandable and non-expandable
  // trees on a page. The options shouldn't be global to all these instances!
  var options;
  var defaultPaddingLeft;

  $.fn.treeTable = function(opts) {
    options = $.extend({}, $.fn.treeTable.defaults, opts);

    return this.each(function() {
      $(this).addClass("treeTable").find("tbody tr").each(function() {
        // Initialize root nodes only if possible
        if(!options.expandable || $(this)[0].className.search(options.childPrefix) == -1) {
          // To optimize performance of indentation, I retrieve the padding-left
          // value of the first root node. This way I only have to call +css+
          // once.
          if (isNaN(defaultPaddingLeft)) {
            defaultPaddingLeft = parseInt($($(this).children("td")[options.treeColumn]).css('padding-left'), 10);
          }

          initialize($(this));
        } else if(options.initialState == "collapsed") {
          this.style.display = "none"; // Performance! $(this).hide() is slow...
        }
      });
    });
  };

  $.fn.treeTable.defaults = {
    childPrefix: "child-of-",
    clickableNodeNames: false,
    expandable: true,
    indent: 19,
    initialState: "collapsed",
    onNodeShow: null,
    treeColumn: 0
  };

  // Recursively hide all node's children in a tree
  $.fn.collapse = function() {
    $(this).addClass("collapsed");

    childrenOf($(this)).each(function() {
      if(!$(this).hasClass("collapsed")) {
        $(this).collapse();
      }

      this.style.display = "none"; // Performance! $(this).hide() is slow...
    });

    return this;
  };

  // Recursively show all node's children in a tree
  $.fn.expand = function() {
    $(this).removeClass("collapsed").addClass("expanded");

    childrenOf($(this)).each(function() {
      initialize($(this));

      if($(this).is(".expanded.parent")) {
        $(this).expand();
      }

      // this.style.display = "table-row"; // Unfortunately this is not possible with IE :-(
      $(this).show();

      if($.isFunction(options.onNodeShow)) {
        options.onNodeShow.call();
      }
    });

    return this;
  };

  // Reveal a node by expanding all ancestors
  $.fn.reveal = function() {
    $(ancestorsOf($(this)).reverse()).each(function() {
      initialize($(this));
      $(this).expand().show();
    });

    return this;
  };

  // Add an entire branch to +destination+
  $.fn.appendBranchTo = function(destination) {
    var node = $(this);
    var parent = parentOf(node);

    var ancestorNames = $.map(ancestorsOf($(destination)), function(a) { return a.id; });

    // Conditions:
    // 1: +node+ should not be inserted in a location in a branch if this would
    //    result in +node+ being an ancestor of itself.
    // 2: +node+ should not have a parent OR the destination should not be the
    //    same as +node+'s current parent (this last condition prevents +node+
    //    from being moved to the same location where it already is).
    // 3: +node+ should not be inserted as a child of +node+ itself.
    if($.inArray(node[0].id, ancestorNames) == -1 && (!parent || (destination.id != parent[0].id)) && destination.id != node[0].id) {
      indent(node, ancestorsOf(node).length * options.indent * -1); // Remove indentation

      if(parent) { node.removeClass(options.childPrefix + parent[0].id); }

      node.addClass(options.childPrefix + destination.id);
      move(node, destination); // Recursively move nodes to new location
      indent(node, ancestorsOf(node).length * options.indent);
    }

    return this;
  };

  // Add reverse() function from JS Arrays
  $.fn.reverse = function() {
    return this.pushStack(this.get().reverse(), arguments);
  };

  // Toggle an entire branch
  $.fn.toggleBranch = function() {
    if($(this).hasClass("collapsed")) {
      $(this).expand();
    } else {
      $(this).removeClass("expanded").collapse();
    }

    return this;
  };

  // === Private functions

  function ancestorsOf(node) {
    var ancestors = [];
    while(node = parentOf(node)) {
      ancestors[ancestors.length] = node[0];
    }
    return ancestors;
  };

  function childrenOf(node) {
    return $("table.treeTable tbody tr." + options.childPrefix + node[0].id);
  };

  function getPaddingLeft(node) {
    var paddingLeft = parseInt(node[0].style.paddingLeft, 10);
    return (isNaN(paddingLeft)) ? defaultPaddingLeft : paddingLeft;
  }

  function indent(node, value) {
    var cell = $(node.children("td")[options.treeColumn]);
    cell[0].style.paddingLeft = getPaddingLeft(cell) + value + "px";

    childrenOf(node).each(function() {
      indent($(this), value);
    });
  };

  function initialize(node) {
    if(!node.hasClass("initialized")) {
      node.addClass("initialized");

      var childNodes = childrenOf(node);

      if(!node.hasClass("parent") && childNodes.length > 0) {
        node.addClass("parent");
      }

      if(node.hasClass("parent")) {
        var cell = $(node.children("td")[options.treeColumn]);
        var padding = getPaddingLeft(cell) + options.indent;

        childNodes.each(function() {
          $(this).children("td")[options.treeColumn].style.paddingLeft = padding + "px";
        });

        if(options.expandable) {
          cell.prepend('<span style="margin-left: -' + options.indent + 'px; padding-left: ' + options.indent + 'px" class="expander"></span>');
          $(cell[0].firstChild).click(function() { node.toggleBranch(); });

          if(options.clickableNodeNames) {
            cell[0].style.cursor = "pointer";
            $(cell).click(function(e) {
              // Don't double-toggle if the click is on the existing expander icon
              if (e.target.className != 'expander') {
                node.toggleBranch();
              }
            });
          }

          // Check for a class set explicitly by the user, otherwise set the default class
          if(!(node.hasClass("expanded") || node.hasClass("collapsed"))) {
            node.addClass(options.initialState);
          }

          if(node.hasClass("expanded")) {
            node.expand();
          }
        }
      }
    }
  };

  function move(node, destination) {
    node.insertAfter(destination);
    childrenOf(node).reverse().each(function() { move($(this), node[0]); });
  };

  function parentOf(node) {
    var classNames = node[0].className.split(' ');

    for(var key=0; key<classNames.length; key++) {
      if(classNames[key].match(options.childPrefix)) {
        return $("#" + classNames[key].substring(9));
      }
    }

    return null;
  };
})(jQuery);
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/

/**
 * Helper functions that may be included and used from pages using
 * inline editing via jeditable.
 */

KT.editable = (function(){
    var common_settings = {
            method          :  'PUT',
            cancel          :  i18n.cancel,
            submit          :  i18n.save,
            indicator       :  i18n.saving,
            tooltip         :  i18n.clickToEdit,
            placeholder     :  i18n.clickToEdit,
            submitdata      :  $.extend({ authenticity_token: AUTH_TOKEN }, KT.common.getSearchParams()),
            onerror         :  function(settings, original, xhr) {
                original.reset();
                $("#notification").replaceWith(xhr.responseText);
                notices.checkNotices();
            }
        },
        initialize = function() {
            initialize_panel_element();
            initialize_ajaxfileupload();
            initialize_password();
            initialize_textfield();
            initialize_textfield_custom_info();
            initialize_textarea();
            initialize_multiselect();
            initialize_select();
            initialize_number();
            initialize_datepicker();
            initialize_timepicker();
        },
        initialize_panel_element = function() {
            $('.edit_panel_element').each(function() {
                $(this).editable('destroy');
                var settings = {
                    type        :  'text',
                    data        :  null,
                    width       :  270,
                    name        :  $(this).attr('name'),
                    onsuccess   :  function(result, status, xhr) {
                        var id = $('#panel_element_id');
                        KT.panel.list.refresh(id.attr('value'), id.attr('data-ajax_url'));
                    }
                };
                $(this).editable($(this).attr('data-url'), $.extend(common_settings, settings));
            });
        },
        initialize_ajaxfileupload = function() {
            $('.ajaxfileupload').each(function() {
                $(this).editable('destroy');
                $(this).editable($(this).attr('url'), {
                    type        :  'ajaxupload',
                    data        :  null,
                    method      :  'PUT',
                    name        :  $(this).attr('name'),
                    cancel      :  i18n.cancel,
                    submit      :  i18n.upload,
                    indicator   :  i18n.uploading,
                    tooltip     :  i18n.clickToEdit,
                    placeholder :  i18n.clickToEdit,
                    submitdata  :  {authenticity_token: AUTH_TOKEN},
                    onerror     :  function(settings, original, xhr) {
                        original.reset();
                        $("#notification").replaceWith(xhr.responseText);
                    }
                });
            });
        },
        initialize_password = function() {
            $('.edit_password').each(function() {
                $(this).editable('destroy');
                var settings = {
                    type        :  'password',
                    data        :  null,
                    width       :  270,
                    name        :  $(this).attr('name')
                };
                $(this).editable($(this).attr('data-url'), $.extend(common_settings, settings));
            });
        },
        initialize_textfield = function() {
            $('.edit_textfield').each(function() {
                $(this).editable('destroy');
                var settings = {
                    type        :  'text',
                    data        :  null,
                    width       :  270,
                    name        :  $(this).attr('name'),
                    onsuccess   : function(result, status, xhr) {
                        element.text(KT.utils.unescape(result)); // hax
                    },
                    onerror     : function(settings, original, xhr) {
                        original.reset();
                    }
                };
                $(this).editable($(this).attr('data-url'), $.extend(common_settings, settings));
            });
        },
        initialize_textfield_custom_info = function() {
            $('.edit_textfield_custom_info').each(function() {
                $(this).editable('destroy');
                var element = $(this);
                var settings = {
                    type        :  'custom_info',
                    data        :  null,
                    width       :  158,
                    name        :  $(this).attr('name'),
                    onsuccess   : function(result, status, xhr) {
                        element.text(result);
                        notices.displayNotice("success", window.JSON.stringify({ "notices": [i18n.custom_info_update_success] }));
                    },
                    onerror     : function(settings, original, xhr) {
                        original.reset();
                        notices.displayNotice("error", window.JSON.stringify({ "notices": [$.parseJSON(xhr.responseText)["displayMessage"]] }));
                    }
                };
                $(this).editable($(this).attr('data-url'), $.extend(common_settings, settings));
            });
        },
        initialize_textarea = function() {
            $('.edit_textarea').each(function() {
                $(this).editable('destroy');
                var element = $(this);
                var settings = {
                    type        :  'textarea',
                    data        :  null,
                    name        :  element.attr('name'),
                    rows        :  8,
                    cols        :  36,
                    maxlength   :  element.data('maxlength'),
                    onsuccess   : function(result, status, xhr) {
                        element.text(KT.utils.unescape(result)); // hax
                    },
                    onerror     : function(settings, original, xhr) {
                        original.reset();
                    }
                };
                $(this).editable($(this).attr('data-url'), $.extend(common_settings, settings));
            });
        },
        initialize_select = function() {
            $('.edit_select').each(function(){
                $(this).editable('destroy');
                var element = $(this);
                var settings = {
                    type            :  'select',
                    name            :  element.attr('name'),
                    data            :  jQuery.proxy(function() { return $(this).data("options"); }, this),
                    onsuccess       :  function(result, status, xhr){
                        var data = element.data('options');

                        data["selected"] = result;
                        element.html(data[result]);
                    }
                };
                $(this).editable($(this).attr('data-url'), $.extend(common_settings, settings));
            });
        },
        initialize_multiselect = function() {
            $('.edit_multiselect').each(function() {
                $(this).editable('destroy');
                var element = $(this);
                var settings = {
                    type            :  'multiselect',
                    name            :  element.attr('name'),
                    data            :  jQuery.proxy(function() { return $(this).data("options"); }, this)
                };
                $(this).editable($(this).attr('data-url'), $.extend(common_settings, settings));
            });
        },
        initialize_number = function() {
            $('.edit_number').each(function() {
                $(this).editable('destroy');
                var element = $(this);
                var settings = {
                    method          :  'POST',
                    type            :  'number',
                    data            :  null,
                    value           :  $.trim($(this).html()),
                    height          :  10,
                    width           :  35,
                    onblur          :  'ignore',
                    name            :  $(this).attr('name'),
                    min             :  $(this).data('min'),
                    max             :  $(this).data('max'),
                    unlimited       :  $(this).data('unlimited'),
                    image           :  $(this).css('background-image'),
                    submitdata      :  {authenticity_token: AUTH_TOKEN},
                    onsuccess       :  function(result, status, xhr){
                        element.css('background-image', settings.image);
                        if ($(this).data('unlimited') !== undefined) {
                            if (parseInt(result,10) === $(this).data('unlimited')) {
                                element.html(i18n.unlimited);
                            } else {
                                element.html(result);
                            }
                        } else {
                            element.html(result);
                        }
                    },
                    onresetcomplete : function(settings, original){
                        element.css('background-image', settings.image);
                    }
                };
                element.editable(element.attr('data-url'), $.extend(common_settings, settings));
            });
        },
        initialize_datepicker = function() {
            $('.edit_datepicker').each(function() {
                $(this).editable('destroy');
                var settings = {
                    type        :  'datepicker',
                    data        :  null,
                    width       :  100,
                    name        :  $(this).attr('name')
                };
                $(this).editable($(this).attr('data-url'), $.extend(common_settings, settings));
            });
        },
        initialize_timepicker = function() {
            $('.edit_timepicker').each(function() {
                $(this).editable('destroy');
                $(this).editable($(this).attr('data-url'), {
                    type        :  'timepicker',
                    data        :  null,
                    width       :  300,
                    method      :  'PUT',
                    name        :  $(this).attr('name'),
                    cancel      :  i18n.cancel,
                    submit      :  i18n.save,
                    indicator   :  i18n.saving,
                    tooltip     :  i18n.clickToEdit,
                    placeholder :  i18n.clickToEdit,
                    submitdata  :  $.extend({ authenticity_token: AUTH_TOKEN }, KT.common.getSearchParams()),
                    onsuccess   :  function(result, status, xhr) {
                        var plan_time = $("#plan_time").text();
                        var current_plan = $("#current_plan").text();
                        if (plan_time !== current_plan) {
                            $("#current_plan").text(plan_time);
                        }
                        var id = $('#plan_id');
                        list.refresh(id.attr('value'), id.attr('data-ajax_url'));
                    },
                    onerror     :  function(settings, original, xhr) {
                      original.reset();
                      $("#notification").replaceWith(xhr.responseText);
                    }
                });
            });
        };

    initialize();

    return {
        initialize                       : initialize,
        initialize_panel_element         : initialize_panel_element,
        initialize_ajaxfileupload        : initialize_ajaxfileupload,
        initialize_password              : initialize_password,
        initialize_textfield             : initialize_textfield,
        initialize_textfield_custom_info : initialize_textfield_custom_info,
        initialize_textarea              : initialize_textarea,
        initialize_select                : initialize_select,
        initialize_multiselect           : initialize_multiselect,
        initialize_number                : initialize_number,
        initialize_datepicker            : initialize_datepicker,
        initialize_timepicker            : initialize_timepicker
    };
}());
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
 */

/*
 * Katello JavaScript Menu File
 * Author: jrist
 * Date: 10/25/11
 * Time: 10:19 AM
 */

/**
 * Document Ready function
 */

$(document).ready(function (){
    $("#tabs").tabs();
});
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/


$(document).ready(function() {


    KT.common.jscroll_init($('.scroll-pane'));
    KT.common.jscroll_resize($('.jspPane'));

    $("#content_tabs").tabs();

    $(".tree_table").treeTable({
        expandable: true,
        initialState: "collapsed",
        clickableNodeNames: true,
        onNodeShow: KT.redhat_provider_page.on_node_show
    });

    $(".content_table").delegate('.expander_area', 'click', function(){
        var area = $(this),
            row = area.parents('tr').first();
        if(row.hasClass("disable")){
            return;
        }
        if(row.hasClass("collapsed")){
            area.parent().find('table').show();
            row.removeClass("collapsed").addClass('expanded');
        }
        else {
            area.parent().find('table').hide();
            row.addClass("collapsed").removeClass('expanded');
        }
    });


    $("#content_tabs").delegate('.repo_set_enable', 'change', function(){
        KT.redhat_provider_page.repoSetChange($(this), false);
    });

    $("#content_tabs").delegate('.repo_enable', 'change', function() {
        KT.redhat_provider_page.repoChange($(this), false);
    });

    $("#content_tabs").delegate('.repo_set_refresh', 'click', function(){
        var icon = $(this),
            row = icon.parents('.repo_set').first(),
            checkbox;

        if(!icon.hasClass('disabled')){
            checkbox = row.find('.repo_set_enable');
            console.log(checkbox);
            KT.redhat_provider_page.repoSetChange(checkbox, true);
        }
    });

});


KT.redhat_provider_page = (function($) {
    var repoChange = function(checkbox) {

        var name = checkbox.attr("name"),
            options = {},
            url = checkbox.attr("data-url"),
            id = checkbox.attr("value"),
            set_checkbox = checkbox.parents(".repo_set").find('.repo_set_enable');

        if (checkbox.attr("checked") !== undefined) {
            options['repo'] = "1";
        } else {
            options['repo'] = "0";
        }

        $(checkbox).hide();
        $('#spinner_'+id).removeClass('hidden').show();
        $.ajax({
            type: "PUT",
            url: url,
            data: options,
            cache: false,
            success: function(data, textStatus, jqXHR){
              KT.redhat_provider_page.checkboxHighlightRow(data['id']);
              if(data['can_disable_repo_set']){
                set_checkbox.removeAttr('disabled');
              }
              else {
                set_checkbox.attr('disabled','disabled');
              }
            }
        });
        return false;
    },
    repoSetChange = function(checkbox, is_refresh) {
        var url = checkbox.data("url"),
            disable_url = checkbox.data("disable_url"),
            content_id = checkbox.attr("value");
        if (checkbox.attr('checked') || is_refresh) {
            refresh_repo_set(url, content_id, is_refresh);
        }
        else {
            disable_repo_set(disable_url, content_id);
        }
    },
    disable_repo_set = function(url, content_id){
        var row = $('#repo_set_' + content_id);
        hide_repos(content_id);
        row.find('.repo_set_enable').hide();
        row.find('.repo_set_spinner').show();
        row.addClass("disable");
        row.find('.repo_set_refresh').addClass('disabled');
        $.ajax({
            type: "PUT",
            url: url,
            data: {content_id:content_id},
            cache: false,
            success: function(data){
                var checkbox = row.find('.repo_set_enable');
                row.find('table').replaceWith('<table style="display: none;"> </table>');
                checkbox.show();
                if (checkbox.data('orphaned')) {
                    checkbox.attr("disabled", "disabled");
                }
                row.find('.repo_set_spinner').hide();
                row.find('.expander').removeClass('disabled').hide();
                row.find('.repo_set_refresh').removeClass('disabled').hide();
            },
            error: function(){
                row.removeClass("disable");
                row.find('.repo_set_enable').show();
                row.find('.repo_set_spinner').hide();
                row.find('.repo_set_refresh').removeClass('disabled');
            }
        });
    },
    refresh_repo_set = function(url, content_id, is_refresh){
        var row = $('#repo_set_' + content_id);
        hide_repos(content_id);
        row.addClass("disable");
        row.find('.repo_set_enable').hide();
        row.find('.repo_set_spinner').show();
        row.find('.repo_set_refresh').addClass('disabled');
        $.ajax({
            type: "PUT",
            url: url,
            data: {content_id:content_id},
            cache: false,
            success: function(data){
                row.find('table').replaceWith(data);
                row.removeClass("disable");
                row.find('.repo_set_enable').show();
                row.find('.repo_set_spinner').hide();
                row.find('.expander').show();
                show_repos(content_id);
                row.find('.repo_set_refresh').removeClass('disabled').show();

            },
            error: function(){
                row.removeClass("disable");
                row.find('.repo_set_enable').show().attr('checked', is_refresh);
                row.find('.repo_set_spinner').hide();
                if(is_refresh){
                    row.find('.repo_set_refresh').removeClass('disabled').hide();
                }
            }
        });
    },
    checkboxHighlightRow = function(id){
      $('#repo-'+id).effect('highlight', {'color':'#390'}, 1000);
      $('#spinner_'+id).hide().addClass('hidden');
      $('#input_repo_'+id).show();
    },
    on_node_show = function(a, b, c){
        $.sparkline_display_visible();
    },
    hide_repos = function(content_id){
        var row = $('#repo_set_' + content_id);
        if(row.hasClass('expanded')){
            row.find('.expander_area').click();
        }
    },
    show_repos = function(content_id){
        var row = $('#repo_set_' + content_id);
        if(row.hasClass('collapsed')){
            row.find('.expander_area').click();
        }
    };

    return {
        repoChange: repoChange,
        checkboxHighlightRow: checkboxHighlightRow,
        on_node_show: on_node_show,
        repoSetChange: repoSetChange,
        hide_repos: hide_repos
    };
}(jQuery));
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/





;
